package dev.fyloz.trial.colorrecipesexplorer;

import dev.fyloz.trial.colorrecipesexplorer.core.io.file.FileHandler;
import dev.fyloz.trial.colorrecipesexplorer.core.services.PasswordService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.MessageSource;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.util.List;

@SpringBootApplication
public class ColorRecipesExplorerApplication {

    public static final Logger LOGGER = LoggerFactory.getLogger(ColorRecipesExplorerApplication.class);
    public static String UPLOAD_LOCATION;
    public static final String USERS_FILE_NAME = "passwords";
    public static boolean USE_PORT;

    public static ColorRecipesExplorerApplication CREApp;

    private MessageSource messageSource;

    public static void main(String[] args) {
        UPLOAD_LOCATION = args[0] != null ? args[0] : "./";

        SpringApplication.run(ColorRecipesExplorerApplication.class, args);
    }

    @Autowired
    public ColorRecipesExplorerApplication(MessageSource messageSource) {
        this.messageSource = messageSource;

        CREApp = this;
        LOGGER.info("Le fichier des utilisateurs se situe à: " + new File(UPLOAD_LOCATION + "/" + USERS_FILE_NAME).getAbsolutePath());

        loadPasswords();
    }

    /**
     * Charge les mots de passes contenus dans le fichier.
     * <p>
     * Un mot de passe correspond à une ligne dans le fichier passwords.txt.
     */
    private void loadPasswords() {
        FileHandler fileHandler = new FileHandler(USERS_FILE_NAME, FileHandler.FileContext.OTHERS, FileHandler.FileExtension.TEXT);
        if (!fileHandler.isValid()) {
            fileHandler.createFile();
        }

        try {
            List<String> fileContent = Files.readAllLines(fileHandler.getPath());

            if (fileContent.size() < 1) {
                LOGGER.warn("Aucun mot de passe trouvé. Il sera impossible d'utiliser certaines fonctionnalitées de l'application.");
            }

            for (String line : fileContent) {
                PasswordService.addPassword(line);
            }
        } catch (IOException e) {
            LOGGER.error("Une erreur est survenue lors du chargement du fichier des utilisateurs", e);
            LOGGER.warn("Il sera impossible d'utiliser certaines fonctionnalitées de l'application.");
        }
    }

    public MessageSource getMessageSource() {
        return messageSource;
    }
}
