package dev.fyloz.trial.colorrecipesexplorer.core.configuration;

import dev.fyloz.trial.colorrecipesexplorer.ColorRecipesExplorerApplication;
import dev.fyloz.trial.colorrecipesexplorer.core.model.MaterialType;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.MaterialTypeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.event.ApplicationReadyEvent;
import org.springframework.context.ApplicationListener;
import org.springframework.core.Ordered;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

import java.util.Optional;

@Component
@Order(Ordered.HIGHEST_PRECEDENCE)
public class InitialDataLoader implements ApplicationListener<ApplicationReadyEvent> {

    private MaterialTypeService materialTypeService;

    @Autowired
    public InitialDataLoader(MaterialTypeService materialTypeService) {
        this.materialTypeService = materialTypeService;
    }

    @Override
    public void onApplicationEvent(ApplicationReadyEvent applicationReadyEvent) {
        if (!materialTypeService.getByName(MaterialType.DEFAULT_MATERIAL_TYPE_NAME).isPresent()) {
            createDefaultMaterialType();
        }
    }

    private void createDefaultMaterialType() {
        MaterialType defaultMaterialType = new MaterialType(MaterialType.DEFAULT_MATERIAL_TYPE_NAME, "", false);

        Optional<MaterialType> optionalSavedMaterialType = materialTypeService.save(defaultMaterialType);
        if (!optionalSavedMaterialType.isPresent()) {
            ColorRecipesExplorerApplication.LOGGER.warn("Échec de la création du type de produit par défaut.");
        }
    }
}
