package dev.fyloz.trial.colorrecipesexplorer.core.io.response;

import dev.fyloz.trial.colorrecipesexplorer.core.model.*;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.stream.Stream;

public enum ResponseDataType {
    MATERIAL("material", Material.class),
    MATERIALS("materials", ArrayList.class, MATERIAL),
    MATERIAL_ID("materialID", Integer.class),
    MATERIAL_CODE("materialCode", String.class),

    MATERIAL_TYPE("materialType", MaterialType.class),
    MATERIAL_TYPES("materialTypes", ArrayList.class, MATERIAL_TYPE),
    MATERIAL_TYPE_NAME("materialTypeName", String.class),

    RECIPE("recipe", Recipe.class),
    RECIPES("recipes", ArrayList.class, RECIPE),
    RECIPE_ID("recipeID", Integer.class),
    RECIPE_CODE("recipeCode", String.class),
    RECIPE_MAP("recipeMap", HashMap.class),

    RECIPE_STEP("recipeStep", RecipeStep.class),
    RECIPE_STEPS("recipeSteps", ArrayList.class, RECIPE_STEP),

    MIX("mix", Mix.class),
    MIXES("mixes", ArrayList.class, MIX),
    MIX_ID("mixID", Integer.class),

    MIX_TYPE("mixType", MixType.class),
    MIX_TYPES("mixTypes", ArrayList.class, MIX_TYPE),

    MIX_QUANTITY("mixQuantity", MixQuantity.class),
    MIX_QUANTITIES("mixQuantities", ArrayList.class, MIX_QUANTITY),

    COMPANY("company", Company.class),
    COMPANIES("companies", ArrayList.class, COMPANY),
    COMPANY_ID("companyID", Integer.class),
    COMPANY_NAME("companyName", String.class),

    IMAGE("image", String.class),
    IMAGES("images", ArrayList.class, IMAGE),

    BLOCK_BUTTON("blockButton", Boolean.class);

    private String dataTypeName;
    private Class dataType;
    private Class listDataType;

    ResponseDataType(String dataTypeName, Class dataType) {
        this(dataTypeName, dataType, null);
    }

    ResponseDataType(String dataTypeName, Class dataType, ResponseDataType listDataType) {
        this.dataTypeName = dataTypeName;
        this.dataType = dataType;

        if (listDataType != null) {
            this.listDataType = listDataType.getDataType();
        }
    }

    public String getDataTypeName() {
        return dataTypeName;
    }

    public Class getDataType() {
        return dataType;
    }

    public Class getListDataType() {
        return listDataType;
    }

    @Override
    public String toString() {
        return getDataTypeName();
    }

    public static ResponseDataType getModelDataTypeFromDataType(Class dataType) {
        return Stream.of(values()).filter(r -> r.getDataType().equals(dataType)).findFirst().orElse(null);
    }
}
