package dev.fyloz.trial.colorrecipesexplorer.core.services.model;

import dev.fyloz.trial.colorrecipesexplorer.core.model.Company;
import dev.fyloz.trial.colorrecipesexplorer.core.services.GenericService;
import dev.fyloz.trial.colorrecipesexplorer.dao.CompanyDao;
import dev.fyloz.trial.colorrecipesexplorer.dao.RecipeDao;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class CompanyService extends GenericService<Company, CompanyDao> {

    private RecipeDao recipeDao;    // Utilise le Dao pour éviter une erreur au démarrage, citant une récursion (CompanyService -> RecipeService -> CompanyService -> ...)

    @Autowired
    public CompanyService(CompanyDao companyDao, RecipeDao recipeDao) {
        super(companyDao);
        this.recipeDao = recipeDao;
    }

    @Override
    public boolean isValidForCreation(Company entity) {
        return super.isValidForCreation(entity) && !existsByName(entity.getCompanyName());
    }

    public boolean existsByName(String name) {
        return dao.existsByCompanyName(name);
    }

    public boolean isLinkedToRecipes(Company company) {
        return !recipeDao.findAllByCompany(company).isEmpty();
    }

    public boolean deleteIfNotLinked(Company company) {
        if (company == null) return false;
        if (isLinkedToRecipes(company)) return false;

        return delete(company);
    }
}
